"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var UserErrorT;
(function (UserErrorT) {
    UserErrorT[UserErrorT["BadIncomingRpcCallId"] = 0] = "BadIncomingRpcCallId";
    UserErrorT[UserErrorT["UnknownComponent"] = 1] = "UnknownComponent";
    UserErrorT[UserErrorT["BadRemoteComponentPropType"] = 2] = "BadRemoteComponentPropType";
    UserErrorT[UserErrorT["RequiredPropMissing"] = 3] = "RequiredPropMissing";
    UserErrorT[UserErrorT["LocalPropMissing"] = 4] = "LocalPropMissing";
})(UserErrorT = exports.UserErrorT || (exports.UserErrorT = {}));
class UserError extends Error {
    constructor(type, humanReadable) {
        super(humanReadable);
        this.type = type;
        this.humanReadable = humanReadable;
    }
}
exports.UserError = UserError;
exports.badIncomingRpcCallId = (id) => {
    return new UserError(UserErrorT.BadIncomingRpcCallId, `Callback with id '${id}' was not found`);
};
exports.unknownComponent = (component) => {
    return new UserError(UserErrorT.UnknownComponent, `Unknown component <${component} />`);
};
exports.requiredPropMissing = (propKey, component) => {
    return new UserError(UserErrorT.RequiredPropMissing, `Required prop '${propKey}' has not been passed to <${component} />`);
};
exports.badPropType = (propKey, prop, component) => {
    return new UserError(UserErrorT.BadRemoteComponentPropType, `Error encoding '${propKey}' with value:\n${prop}\npassed to <${component} />`);
};
exports.localPropMissing = (propKey, component) => {
    return new UserError(UserErrorT.LocalPropMissing, `Local prop '${propKey}' has not been passed to <${component} />`);
};
